{{
*****************************************
* File jvmDefines: constant definitions *
*****************************************
}}


CON

  'Virtual Peripheral types
  NONE   = 0
  TIMER  = 1
  ADC    = 2
  DAC    = 3
  PWM    = 4
  UARTRX = 5
  UARTTX = 6

  MAX_NUM_VPS = 6
  
  '// The SX banks the VPs use.
  nmPeriph0Bank = $80
  nmPeriph1Bank = nmPeriph0Bank + $10
  nmPeriph2Bank = nmPeriph0Bank + $20
  nmPeriph3Bank = nmPeriph0Bank + $30
  nmPeriph4Bank = nmPeriph0Bank + $40
  nmPeriph5Bank = nmPeriph0Bank + $50

  'vector register (offsets within a VP bank) 
  nmPeriphISRVec = $00  'each bank, holds VP type

  'propeller fields for pinmasks in vpRambank slots
  nmInmask = 16    'input pin 
  nmOutmask = 20   'output pin
  
  
CON
  'file stamp\core\Timer.java (offsets within a VP bank)
  nmLatchTimer1  = $06  '//32bit latch register
  nmLatchTimer2  = $07
  nmLatchTimer3  = $08
  nmLatchTimer4  = $09  
  nmTimer1       = $02  '//32bit free running timer, incremented every 8.68 microsecond
  nmTimer2       = $03
  nmTimer3       = $04
  nmTimer4       = $05


CON
  'file stamp\core\ADC.java (offsets within a VP bank) 
  nmAdcValue = $0B  '// Current ADC value.
  nmInPort   = $0C  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmInPin    = $0D  '// bitmask (1 of 8 set)
  nmOutPort  = $0E  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmOutPin   = $0F  '// bitmask (1 of 8 set)
  'unused registers used for accumulator, count and pin masks
  nmAdcAccu    = $0A
  nmAdcCount   = $01
  'nmAdcInMask  = $02  'long-aligned input pin mask
  'nmAdcOutMask = $06  'long-aligned output pin mask
  
CON
  'file stamp\core\DAC.java (offsets within a VP bank) 
  nmDacValue = $0B  '// Current DAC value (dutycycle).
  nmDacPort  = $0E  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmDacPin   = $0F  '// bitmask (1 of 8 set)
  'unused registers used for accumulator and output pin mask
  nmDacAccu    = $0A
  'nmDacOutMask = $06 '//long-aligned output pin mask
  

CON
  'file stamp\core\PWM.java (offsets within a VP bank) 
  nmPWMPort     = $0E  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmPWMPin      = $0F  '// bitmask (1 of 8 set)
  'unused registers used for count registers and output pin mask
  nmPWMhighTime = $02
  nmPWMlowTime  = $03
  nmPWMhighZero = $04
  nmPWMlowZero  = $05
  'nmPWMoutMask  = $06 '//long-aligned output pin mask 
  nmPWMCount    = $0A
  nmPWMState    = $0B
  
    
CON
  'file stamp\core\Uart.java (offsets within a VP bank) 
  nmUartDataH    = $02  '// transmit register highbyte
  nmUartDataL    = $03  '// transmit register lowbyte 
  nmUartInvert   = $05  '// 0 for normal, 1 for invert
  nmUartBaud     = $06  '// 1000000/(bps * 4.34)
  nmUartStopBits = $07  '// Tx only: 00000001 for 1 stop bit, 00000011 for 2 etc
  nmUartPort     = $0C  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmUartPin      = $0D  '// bit mask (1 of 8 set)
  nmUartHSPort   = $0E  '// 0 for PortA, 1 for PortB (reserved: 2 for PortC, 3 for PortD)
  nmUartHSPin    = $0F  '// bitmask (1 of 8 set)
  nmUartBufferH  = $0A  '// object reference of 256 byte buffer (so buffer starts at offset +4) 
  nmUartBufferL  = $0B
  nmUartHead     = $08  '//Index of the next free byte in the buffer. 
  nmUartTail     = $09  '//Index of the oldest byte in the buffer.
                        '//If tailPointer == headPointer the buffer is empty.
  'unused registers for count and divide
  nmUartCount    = $01
  nmUartDivide   = $04
  
  'Uart baud constants (divide values)
  speed600   = 192  'The speed value for 600 baud.
  speed1200  = 96   'The speed value for 1200 baud.
  speed2400  = 48   'The speed value for 2400 baud.
  speed4800  = 24   'The speed value for 4800 baud.
  speed7200  = 16   'The speed value for 7200 baud.
  speed9600  = 12   'The speed value for 9600 baud.
  speed14400 = 8    'The speed value for 14400 baud.
  speed19200 = 6    'The speed value for 19200 baud.
  speed28800 = 4    'The speed value for 28800 baud.
  speed38400 = 3    'The speed value for 38400 baud.
  speed57600 = 2    'The speed value for 57600 baud.


PUB dummy
  