{{
*******************************************************
* File jvmSubReturn: handling subroutines and returns *
*******************************************************
}}

{{
  Instructions that jump to a subroutine push the return address onto the stack.
  This address is retrieved from a local variable.

  Returning from Methods.  
  Returns are handled in one of two ways by bytecode instructions: return (void) or return (normal). 
  Void returns are used to back out cleanly from the previous method. Thus, all values on the operand stack 
  are discarded and the interpreter then returns control to its caller. Normal returns are used to proceed 
  normally with execution and push a value associated with the previous method onto the stack. After the 
  return value is pushed onto the stack, any other values on the operand stack are discarded and the 
  interpreter then returns control to its caller. 
  Note:
  Java's operand stack is not contiguous like the operand stacks you may be familiar with from programming
  in other languages. Because each method has its own section of the operand stack, when the operand stack
  for the method is discarded, only the section related to the method is cleared out. 
}}


OBJ
  js:   "jvmStack"
  jd:   "jvmData"
  
  
PUB jb_jsr | offset
{{ Meaning: Jump subroutine. }}
{{
  Description: Execution proceeds at the signed 16-bit offset from the address of this instruction.
  The offset is constructed from byte1 and byte2. The address of the instruction immediately following
  the current instruction is pushed onto the stack to provide a return address from the subroutine.
  The jsr instruction is used with the ret instruction in the implementation of the finally clauses of the
  Java programming language. Note that jsr pushes the address onto the operand stack and ret gets it out
  of a local variable. This asymmetry is intentional.
}}
{{
  JEM format: jsr,byte1,byte2
}}
  offset := jd.readJavaIntBE(jd.getJvmPC+1)
  js.push(jd.getJvmPC+3)
  jd.setJvmPC(jd.getJvmPC + offset)
  

{{PUB jb_jsr_w}}
{{ Meaning: Jump subroutine for wide index. }}
{{
  Description: Execution proceeds at the signed 32-bit offset from the address of this instruction.
  The offset is constructed from byte1, byte2, byte3, and byte4. The address of the instruction immediately
  following the current instruction is pushed onto the stack to provide a return address from the subroutine.
}}


PUB jb_ret | index,retaddr
{{ Meaning: Return from subroutine. }}
{{
  Description: Local variable in the current Java frame must contain a return address.
  The contents of the local variable are written into the program counter.
  The ret instruction is used with jsr or jsr_w instructions in the implementation of the finally clauses of the
  Java programming language. Note that jsr pushes the address onto the operand stack and ret gets it out of a
  local variable. This asymmetry is intentional. The ret instruction should not be confused with the return
  instruction. A return instruction returns control from a method to its invoker, without passing any value back
  to the invoker. The ret opcode can be used in conjunction with the wide instruction to access a local variable
  using a two-byte unsigned index.
}}
{{
  JEM format: ret,byte1
}}
  index := jd.readJavaByte(jd.getJvmPC+1)
  retaddr := jd.readJavaIntBE(jd.getJvmFP + (index<<1))
  jd.setJvmPC(retaddr)
  

{{PUB jb_ret_w}}
{{ Meaning: Return from subroutine for wide index. }}
{{
  Description: Local variable in the current Java frame must contain a return address.
  The contents of the local variable are written into the program counter.
}}


{{PUB jb_breakpoint}}
{{ Meaning: Breakpoint. }}
{{
  Description: Stop and pass control to breakpoint handler.
}}


PUB jb_return | method,size,sp,fp
{{ Meaning: Return (void) from method. }}
{{
  JEM format: return
}}
  method := jd.getJvmMP                          'current method
  size := (jd.readJavaByte(method + jd#JVM_PARAMETERS) & $7F) + jd.readJavaByte(method + jd#JVM_LOCALS) 'offset to saved mp,fp,pc in current frame
  sp := jd.getJvmFP + size
  jd.setJvmMP(jd.readJavaIntBE(sp))              'restore mp     
  fp := jd.readJavaIntBE(sp+2)
  jd.setJvmFP(fp)                                'restore fp
  jd.setJvmPC(jd.readJavaIntBE(sp+4))            'restore pc
  sp -= size
  jd.setJvmSP(sp)                                'restore sp
    

PUB jb_ireturn | val
{{ Meaning: Return integer from function. }}
{{
  JEM format: ireturn
}}
  val := js.pop                               'pop return value
  jb_return                                      'discard frame
  js.push(val)                                'push return value


{{PUB jb_lreturn}}
{{ Meaning: Return long integer from function. }}


{{PUB jb_freturn}}
{{ Meaning: Return single float from function. }}


{{PUB jb_dreturn}}
{{ Meaning: Return double float from function. }}


{{PUB jb_areturn}}
{{ Meaning: Return object reference from function. }}

