{{
**********************************************************
* File jvmVirtualPeripheral: native routines for VP code *
**********************************************************
}}

{{
  Virtual Peripherals are implemented as background code, meaning the code executes
  independantly from the interpreted java code.
}}


OBJ
  def:    "jvmDefines"
  vpCog1: "jvmVpCore"
  vpCog2: "jvmVpCore"
  vpQ:    "jvmVpQueue"
  
  
DAT
  'do not change order
  vpTimerCount  long 0                       'holds 32bit free running timer
  vpRambank     long 0[def#MAX_NUM_VPS*6]    '6 longs per VP slot, first 16 bytes are VP registers, last 2 longs are input pinmask and output pinmask
  vpQueueStack  long 0[20]
  vpQid         long 0
  

CON
  firstBankVPcog1 = 1*4
  firstBankVPcog2 = firstBankVPcog1 + ((def#MAX_NUM_VPS*6)*4/2)

    
PUB start
{{ startup COG's for VP code }}
  vpTimerCount := firstBankVPcog1  'first bank of registers for vpCog1 is at 54 bytes offset to vpTimerCount
  vpCog1.start(@vpTimerCount)
  vpTimerCount := firstBankVPcog2  'first bank of registers for vpCog2 is at 102 bytes offset to vpTimerCount
  vpCog2.start(@vpTimerCount)
  vpQid := cognew(vpQ.start(@vpRambank),@vpQueueStack)+1 'run queue code in seperate cog
  
  
PUB stop
{{ stop COG's for VP code }}
  cogstop(vpQid-1)
  vpCog2.stop
  vpCog1.stop
  
  
PUB writeVPregister(address,value) | bank,offset
{{ write value to vpRambank register }}
  bank := (address & $F0)
  offset := (address & $0F)
  byte[@vpRambank + (((bank - def#nmPeriph0Bank)*3)/2) + offset] := value
  
  
PUB readVPregister(address):value | bank,offset
{{ read value from vpRambank register }}
  bank := (address & $F0)
  offset := (address & $0F)
  return byte[@vpRambank + (((bank - def#nmPeriph0Bank)*3)/2) + offset] & $FF


PUB nm_core_Timer_init(bank)
  '/**
  ' * Initialize the 32bit counter. The 32bit nmTimer register should increment every 8.68 usec.
  ' * Called by Timer.start()
  ' *
  ' * @param bank The allocated area for the Timer registers.
  ' */
  vpTimerCount := 0
  writeVPregister(bank + def#nmPeriphISRVec, def#TIMER)
  

PUB nm_core_Timer_latch(bank) | latchedCount
  '/**
  ' * Latch the 32bit counter into the latch registers.
  ' * Called by Timer.tickLo()
  ' *
  ' * @param bank The allocated area for the Timer registers.
  ' */
  latchedCount := vpTimerCount                                  'get current count 
  writeVPregister(bank + def#nmLatchTimer1, latchedCount      ) 'write b0
  writeVPregister(bank + def#nmLatchTimer2, latchedCount >> 8 ) 'write b1
  writeVPregister(bank + def#nmLatchTimer3, latchedCount >> 16) 'write b2
  writeVPregister(bank + def#nmLatchTimer4, latchedCount >> 24) 'write b3
  

PUB nm_core_ADC_install(bank) | slot,pin,port,pinmask
  '/**
  ' * Initialize an ADC as background task. Its parameters are in vpRambank.
  ' * Called by ADC.start()
  ' *
  ' * @param bank The allocated area for the ADC registers.
  ' */
  slot := (bank - def#nmPeriph0Bank) >> 4 'bank $80..$D0 converted to slot 0..5
  pin := readVPregister(bank + def#nmInPin)
  port := readVPregister(bank + def#nmInPort)
  if pin == 0
    pinmask := 0
  else
    pinmask := pin << (port<<3)
  vpRambank[slot*6 + 4] := pinmask
  pin := readVPregister(bank + def#nmOutPin)
  port := readVPregister(bank + def#nmOutPort)
  if pin == 0
    pinmask := 0
  else
    pinmask := pin << (port<<3)
  vpRambank[slot*6 + 5] := pinmask
  writeVPregister(bank + def#nmPeriphISRVec, def#ADC) 'start adc VP

  
PUB nm_core_DAC_install(bank) | slot,pin,port,pinmask
  '/**
  ' * Initialize a DAC as background task. Its parameters are in vpRambank.
  ' * Called by DAC.start()
  ' *
  ' * @param bank The allocated area for the DAC registers.
  ' */
  slot := (bank - def#nmPeriph0Bank) >> 4 'bank $80..$D0 converted to slot 0..5
  vpRambank[slot*6 + 4] := 0 'no input pin
  pin := readVPregister(bank + def#nmDacPin)
  port := readVPregister(bank + def#nmDacPort)
  if pin == 0
    pinmask := 0
  else
    pinmask := pin << (port<<3)
  vpRambank[slot*6 + 5] := pinmask
  writeVPregister(bank + def#nmDacAccu, 0)
  writeVPregister(bank + def#nmPeriphISRVec, def#DAC) 'start dac VP


PUB nm_core_PWM_install(bank) | slot,pin,port,pinmask
  '/**
  ' * Initialize a PWM as background task. Its parameters are in vpRambank.
  ' * Called by PWM.start()
  ' *
  ' * @param bank The allocated area for the PWM registers.
  ' */
  slot := (bank - def#nmPeriph0Bank) >> 4 'bank $80..$D0 converted to slot 0..5
  vpRambank[slot*6 + 4] := 0 'no input pin
  pin := readVPregister(bank + def#nmPWMPin)
  port := readVPregister(bank + def#nmPWMPort)
  if pin == 0
    pinmask := 0
  else
    pinmask := pin << (port<<3)
  vpRambank[slot*6 + 5] := pinmask
  writeVPregister(bank + def#nmPeriphISRVec, def#PWM) 'start pwm VP


PUB nm_core_PWM_updateInternal(bank,highTime,lowTime,highZero,lowZero)
  '/**
  ' * Update the PWM duty cycle. Its parameters are in vpRambank. 
  ' * Called by PWM.update()
  ' *
  ' * @param bank The allocated area for the PWM registers.
  ' * @param highTime the number of 8.68us the waveform should be high. (1-255)
  ' * @param lowTime the number of 8.68us the waveform should be low. (1-255)
  ' * @param highZero Boolean value (always true) that tells wether highTime != 0.
  ' * @param lowZero Boolean value (always true) that tells wether lowTime != 0.
  ' */
  writeVPregister(bank + def#nmPWMlowTime, lowTime)
  writeVPregister(bank + def#nmPWMhighTime, highTime)
  writeVPregister(bank + def#nmPWMlowZero, lowZero)
  writeVPregister(bank + def#nmPWMhighZero, highZero)
       

PUB nm_core_Uart_rxInit(bank) | bufptr,slot,pin,port,pinmask,invert,stopBits
  '/**
  ' * Initialize a Uart receiver as background task. Its parameters are in vpRambank.
  ' * Called by Uart.start()
  ' *
  ' * @param bank The allocated area for the Uart registers.
  ' */
  slot := (bank - def#nmPeriph0Bank) >> 4 'bank $80..$D0 converted to slot 0..5
  pin := readVPregister(bank + def#nmUartPin)
  port := readVPregister(bank + def#nmUartPort)
  if pin == 0
    pinmask := 0
  else
    pinmask := pin << (port<<3)
  vpRambank[slot*6 + 4] := pinmask
  pin := readVPregister(bank + def#nmUartHSPin)
  port := readVPregister(bank + def#nmUartHSPort)
  if pin == 0
    pinmask := 0
  else
    pinmask := pin << (port<<3)
  vpRambank[slot*6 + 5] := pinmask

  bufptr := (readVPregister(bank + def#nmUartBufferH)<<8) + readVPregister(bank + def#nmUartBufferL) 'get buffer object reference
  bufptr += 4 'we do not need to check for null pointer, as a NullPointerException would be thrown before we get here
  writeVPregister(bank + def#nmUartBufferH, bufptr)    'store buffer address as little endian
  writeVPregister(bank + def#nmUartBufferL, bufptr>>8)

  stopBits := >| readVPregister(bank + def#nmUartStopBits) 'nmUartStopBits is 1,3,7,15...255 so stopBits is 0 to 7
  stopBits += 10  'make it 1 to 8 stopbits + startbit + 8 databits
  writeVPregister(bank + def#nmUartStopBits, stopBits) 'store stopBits (=total bits) as decimal value
  writeVPregister(bank + def#nmPeriphISRVec, def#UARTRX)


PUB nm_core_Uart_rxRead(bank) | slot,pinmask,head,tail,len,invert
  '/**
  ' * Inform Uart receiver a byte has been grabbed from the buffer, so the receiver can do the handshaking.
  ' * Called by Uart.readByte()
  ' *
  ' * @param bank The allocated area for the Uart registers.
  ' */
  slot := (bank - def#nmPeriph0Bank) >> 4 'bank $80..$D0 converted to slot 0..5
  pinmask := vpRambank[slot*6 + 5] 'get HS pinmask
  if pinmask <> 0
    invert := readVPregister(bank + def#nmUartInvert) & $02
    head := readVPregister(bank + def#nmUartHead)
    tail := readVPregister(bank + def#nmUartTail)
    len := (head - tail + 256) // 256
    if len =< 16
      'enable hsPortpin (tell remote sender to resume transmitting)
      if invert
        outa[pinmask]~~
      else
        outa[pinmask]~
    if len => 240
      'disable hsPortPin (tell remote sender to stop transmitting)
      if invert
        outa[pinmask]~
      else
        outa[pinmask]~~
    dira[pinmask]~~     'make pin output
   
      
PUB nm_core_Uart_txInit(bank) | bufptr,slot,pin,port,pinmask,invert,stopBits
  '/**
  ' * Initialize a Uart transmitter as background task. Its parameters are in vpRambank.
  ' * Called by Uart.start()
  ' *
  ' * @param bank The allocated area for the Uart registers.
  ' */
  slot := (bank - def#nmPeriph0Bank) >> 4 'bank $80..$D0 converted to slot 0..5
  pin := readVPregister(bank + def#nmUartHSPin)
  port := readVPregister(bank + def#nmUartHSPort)
  if pin == 0
    pinmask := 0
  else
    pinmask := pin << (port<<3)
  vpRambank[slot*6 + 4] := pinmask
  pin := readVPregister(bank + def#nmUartPin)
  port := readVPregister(bank + def#nmUartPort)
  if pin == 0
    pinmask := 0
  else
    pinmask := pin << (port<<3)
  vpRambank[slot*6 + 5] := pinmask

  bufptr := (readVPregister(bank + def#nmUartBufferH)<<8) + readVPregister(bank + def#nmUartBufferL) 'get buffer object reference
  bufptr += 4 'we do not need to check for null pointer, as a NullPointerException would be thrown before we get here
  writeVPregister(bank + def#nmUartBufferH, bufptr)    'store buffer address as little endian
  writeVPregister(bank + def#nmUartBufferL, bufptr>>8)

  stopBits := >| readVPregister(bank + def#nmUartStopBits) 'nmUartStopBits is 1,3,7,15...255 so stopBits is 0 to 7
  stopBits += 10  'make it 1 to 8 stopbits + startbit + 8 databits
  writeVPregister(bank + def#nmUartStopBits, stopBits) 'store stopBits (=total bits) as decimal value
  writeVPregister(bank + def#nmPeriphISRVec, def#UARTTX)

  