{{
**************************************************
* File jvmVpCore: VP code (runs in seperate COG) *
**************************************************
}}

{{
  Virtual Peripherals are implemented as background code, meaning the code executes
  independantly from the interpreted java code.
}}


OBJ
  def: "jvmDefines"


VAR
  long vpId

  
PUB start(timerAddr)
{{ startup pasm routine }}
  vpId := cognew(@entry,timerAddr)+1
  if vpId > 0
    repeat while long[timerAddr] <> 0  'wait until COG initialized


PUB stop
{{ stop pasm routine }}
  if vpId > 0
    cogstop(vpId-1)

  
DAT
                        org     0
entry
                        mov     asmTimerCountAddr, PAR              ' 4  get address of vpTimerCount 
                                                                    '12 
                        rdlong  asmVpRambankArea, asmTimerCountAddr ' 7  get value stored in vpTimerCount (= offset to VP rambank area for this COG)
                        add     asmVpRambankArea, asmTimerCountAddr ' 4  add to address of vpTimerCount
                        mov     asmBankOffset, #0                   ' 4  clear bankOffset
                                                                    ' 1
                        wrlong  asmBankOffset, asmTimerCountAddr    ' 7  clear vpTimerCount (signal cog is initialized)
 
isrAsm                  'forever loop for VP code
                        mov     time, cnt                           ' 4  record current time
                        add     time, period                        ' 4  establish next period
                        mov     vpInput,ina                         ' 4  sample inputs
                        mov     vpIn, #0                            ' 4  clear input mask
                        mov     vpOutLow, #0                        ' 4  clear output masks
                        mov     vpOutHigh,#0                        ' 4
                        mov     asmRegBase, asmVpRambankArea        ' 4  calculate regBase
                        mov     adrInmask, asmVpRambankArea         ' 4
                        add     adrInmask, #def#nmInmask            ' 4
                        mov     adrOutmask, asmVpRambankArea        ' 4
                        add     adrOutmask, #def#nmOutmask          ' 4
                        mov     vpSlotCount, #def#MAX_NUM_VPS/2     ' 4
                        
                'inner loop
                'total max cycles = 3*(160+44) = 3*204 = 612
isrAsm_nextSlot         'get VP type from next VP slot and execute appropiate VP code
                        rdbyte  asmVpType, asmRegBase               ' 7  regBase + 0 holds vpType or 0 if VP slot free
                        add     asmVpType, #vpCoreJumpTable         ' 4  calculate address holding address to jump to
                        movs    isrAsm_setJump, asmVpType           ' 4  enable to read this address
isrAsm_setJump          movd    isrAsm_dojump, 0-0                  ' 4  put it in place
isrAsm_dojump           jmp     0-0                                 ' 4  jump to appropiate VP code (=< 160 cycles)
isrAsm_retFromVP   'all VP code returns here
                        'move to next VP slot
                        add     adrInmask, #24                      ' 4
                        add     adrOutmask, #24                     ' 4
                        add     asmRegBase, #24                     ' 4
                        sub     vpSlotCount, #1 wz                  ' 4  we do 3 VP slots in this COG
              if_z      jmp     #isrAsm_nextSlot                    ' 4

                'outer loop
                'total cycles = 612 + 72 = 684                      ' less than 694 so we should be ok, even for 3 uarts
isrAsm_waitForTick      'wait for next 8.68 usec tick
                        waitcnt time, period                        ' this is our tickstart
                        andn    dira, vpIn                          ' 4  make pins input
                        andn    outa, vpOutLow                      ' 4  set output latches low
                        or      outa, vpOutHigh                     ' 4  set output latches high
                        or      dira, vpOutLow                      ' 4  make pins output
                        or      dira, vpOutHigh                     ' 4
                        jmp     #isrAsm                             ' 4  loop around
                      

vpNoneAsm     'Cycles is 111 every loop
                        'wait an average time to minimize jitter
                        mov     asmCount, cnt           ' 4
                        add     asmCount, #103          ' 4
                        waitcnt asmCount, #103          ' 103-4
vpNoneAsm_ret           jmp     #isrAsm_retFromVP       ' 4


vpTimerAsm    'Cycles is 27 every loop
                        'increment free running 32bit timer
                        rdlong  asmCount, asmTimerCountAddr ' 7
                        add     asmCount, #1                ' 4
                        nop                                 ' 4
                                                            ' 1
                        wrlong  asmCount, asmTimerCountAddr ' 7
vpTimerAsm_ret          jmp     #isrAsm_retFromVP           ' 4


vpAdcAsm      'Cycles is 139 every loop
                        'count high input at inpin over 256 loops
                        rdlong  asmInMask, adrInmask         ' 7  get pinmask for input pin
                        mov     asmInput, vpInput            ' 4  get inputs
                        and     asmInput, asmInMask wz       ' 4  extract level at input pin
                                                             ' 1
                        rdlong  asmOutMask, adrOutmask       ' 7  get pinmask for output pin
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmAdcAccu   ' 4
                                                             ' 1
                        rdbyte  asmAccu, asmRegAddr          ' 7  get accu
              if_nz     or      vpOutLow, asmOutMask         ' 4  if input high then make outpin low
              if_z      or      vpOutHigh, asmOutMask        ' 4  if input low then make output high ...
              if_z      add     asmAccu, #1                  ' 4  ... and increment accu
              if_z      and     asmAccu, #$FF wz             ' 4  ... test for overflow (zero if overflow)
              if_z      mov     asmAccu, #255                ' 4  ... prevent overflow
                        sub     asmRegAddr, #def#nmAdcAccu - def#nmAdcCount  ' 4
                                                             ' 1
                        rdbyte  asmCount, asmRegAddr         ' 7  get loop count
                        sub     asmCount, #255 wz            ' 4  test for 256 loops
                        add     asmCount, #256               ' 4  increment loop count
                                                             ' 1
                        wrbyte  asmCount, asmRegAddr         ' 7  set loop count
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmAdcValue  ' 4
                                                             ' 1
                        rdbyte  asmValue, asmRegAddr         ' 7  asmValue := adcValue
              if_z      mov     asmValue, asmAccu            ' 4  if 256 loops asmValue := asmAccu
              if_z      mov     asmAccu, #0                  ' 4  if 256 loops then clear accu
                                                             ' 1
                        wrbyte  asmValue,asmRegAddr          ' 7  adcValue := asmValue
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmAdcAccu   ' 4
                                                             ' 1
                        wrbyte  asmRegAddr,asmAccu           ' 7  set accu
                        or      vpIn, asmInmask              ' 4  make inpin input
vpAdcAsm_ret            jmp     #isrAsm_retFromVP            ' 4


vpDacAsm      'Cycles is 79 every loop
                        'make outpin low or high based on accu overflow
                        rdlong  asmOutMask, adrOutmask       ' 7  get pinmask for output pin
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmDacAccu   ' 4
                                                             ' 1
                        rdbyte  asmAccu, asmRegAddr          ' 7  get accu
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmDacValue  ' 4
                                                             ' 1
                        rdbyte  asmValue, asmRegAddr         ' 7  get dacValue
                        add     asmAccu, asmValue            ' 4  accu += dacValue
                        sub     asmRegAddr, #def#nmDacValue - def#nmDacAccu ' 4
                                                             ' 1
                        wrbyte  asmAccu, asmRegAddr          ' 7  set accu
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmDacPin    ' 4
                        shr     asmAccu, #8 wz               ' 4  test accu overflow (eg. > 255 ? )
              if_z      or      vpOutLow, asmOutMask         ' 4  if not overflow make output low
              if_nz     or      vpOutHigh, asmOutMask        ' 4  if overflow make output high
vpDacAsm_ret            jmp     #isrAsm_retFromVP            ' 4


vpPwmAsm      'Cycles is 119 every loop
                        'make outpin low or high based on counters
                        rdlong  asmOutMask, adrOutmask       ' 7  get pinmask for output pin
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmPWMState  ' 4
                                                             ' 1
                        rdbyte  asmState, asmRegAddr         ' 7  get state
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmPWMCount  ' 4
                                                             ' 1
                        rdbyte  asmCount, asmRegAddr         ' 7  get count for current state
                        sub     asmCount, #1 wz              ' 4  decrement count and test for zero
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmPWMhighTime ' 4
                        add     asmRegAddr, asmState         ' 4
              if_z      xor     asmState, #1                 ' 4  if count==0 toggle state
                        nop                                  ' 4
                                                             ' 1
                        rdbyte  asmValue, asmRegAddr         ' 7  get new count
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmPWMState  ' 4
                                                             ' 1
                        wrbyte  asmState, asmRegAddr         ' 7  set state
              if_z      mov     asmCount, asmValue           ' 4  if count==0 load new count
                        sub     asmRegAddr, #def#nmPWMState - def#nmPWMCount ' 4
                                                             ' 1
                        wrbyte  asmCount, asmRegAddr         ' 7  set count
                        and     asmState, #1 wz              ' 4
              if_z      or      vpOutLow, asmOutMask         ' 4  if state==0 make output low
              if_nz     or      vpOutHigh, asmOutMask        ' 4  if state==1 make output high
vpPwmAsm_ret            jmp     #isrAsm_retFromVP            ' 4


{{
  Uart Receive VP code
  For receive, receive bits and when a complete byte is received set a flag.
}}
vpUartRxAsm   'Cycles while receiving but counting down divisor: 91
              'Cycles while receiving and bit sampling: 155
              'Cycles while not receiving: 139
                        'receive a bit of data 
                        rdlong  asmInMask, adrInmask         ' 7  get pinmask for input pin
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmUartBaud  ' 4
                                                             ' 1
                        rdbyte  asmBaud, asmRegAddr          ' 7  get baud
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmUartCount ' 4
                                                             ' 1
                        rdbyte  asmCount, asmRegAddr wz      ' 7  get count, receiving byte?
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmUartDivide ' 4
                                                             ' 1
                        rdbyte  asmDivide, asmRegAddr        ' 7  get divide
              if_nz     jmp     #vpUartRxAsm_rxbit1          ' 4  if receiving byte, jump ahead
                        'at this point count==0
                        add     asmRegAddr, #def#nmUartInvert - def#nmUartDivide ' 4
                                                             ' 1
                        rdbyte  asmInvert, asmRegAddr        ' 7  get invert
                        sub     asmRegAddr, #def#nmUartInvert - def#nmUartDivide ' 4
                        and     asmInvert, #1 wz             ' 4  extract invert state for data
                        mov     asmInput, vpInput            ' 4  get inputs
              if_nz     xor     asmInput, asmInmask          ' 4  invert if required
                        and     asmInput, asmInmask wz       ' 4  extract input bit
              if_z      mov     asmCount, #10                ' 4  if startbit, renew bitcount
                        mov     asmDivide, asmBaud           ' 4  
                        shr     asmDivide, #1                ' 4  ready half bittime to sample startbit
                        nop                                  ' 4
vpUartRxAsm_rxbit1      sub     asmDivide, #1 wz             ' 4  middle of next bit? 
                                                             ' 1
                        wrbyte  asmDivide, asmRegAddr        ' 7  set divide
              if_nz     jmp     #vpUartRxAsm_rxbit2          ' 4  if not in middle of bit jump ahead
                        'at this point we received a new bit
                        sub     asmRegAddr, #def#nmUartDivide - def#nmUartDataH ' 4
                                                             ' 1
                        rdword  asmValue, asmRegAddr         ' 7  get received data
                        and     asmInput, asmInMask wz       ' 4  extract level at input pin
              if_nz     or      asmValue, #$100              ' 4  shift in new bit
                        sub     asmCount, #1 wz              ' 4  did we receive last bit ?
              if_z      or      asmValue, #$100              ' 4  if so, mark byte received
              if_nz     shr     asmValue, #1                 ' 4  if not, move to right
                        nop                                  ' 4
                                                             ' 1
                        wrword  asmValue, asmRegAddr         ' 7  save bits
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmUartDivide ' 4
                                                             ' 1
                        wrbyte  asmBaud, asmRegAddr          ' 7  ready bittime
                        nop                                  ' 4
vpUartRxAsm_rxbit2      sub     asmRegAddr, #def#nmUartDivide - def#nmUartCount ' 4
                                                             ' 1
                        wrbyte  asmCount, asmRegAddr         ' 7  set count
vpUartRxAsm_ret         jmp     #isrAsm_retFromVP            ' 4  done


{{
  Uart Transmit VP code
  For transmit, if the nmUartCount variable <> 0 then we are transmitting a byte.
}}
vpUartTxAsm   'Cycles while counting down divisor: 59
              'Cycles while reloading divisor and not transmitting: 75
              'Cycles while reloading divisor and transmitting: 159
                        'transmit a bit of data 
                        rdlong  asmOutMask, adrOutmask       ' 7  get pinmask for output pin (data)
                        'decrement divide
                        mov     asmRegAddr, asmRegBase       ' 4  get divide
                        add     asmRegAddr, #def#nmUartDivide ' 4
                                                             ' 1
                        rdbyte  asmDivide, asmRegAddr        ' 7
                        sub     asmDivide, #1 wz             ' 4  decrement divisor and test for zero
                        add     asmRegAddr, #def#nmUartBaud - def#nmUartDivide ' 4  divide := vpRambank[bank+nmUartBaud]
                                                             ' 1
                        rdbyte  asmValue, asmRegAddr         ' 7
                        sub     asmRegAddr, #def#nmUartBaud - def#nmUartDivide ' 4
              if_z      mov     asmDivide, asmValue          ' 4  reload divisor
                                                             ' 1
                        wrbyte  asmDivide, asmRegAddr        ' 7  set divide
              if_nz     jmp     #isrAsm_retFromVP            ' 4  done
                        'we only get here if divide is reloaded
                        sub     asmRegAddr, #def#nmUartDivide - def#nmUartCount ' 4
                                                             ' 1
                        rdbyte  asmCount, asmRegAddr wz      ' 7  get count
              if_z      jmp     #isrAsm_retFromVP            ' 4  done 
                        'we only get here if we are transmitting a byte
                        sub     asmCount, #1 wz              ' 4  dec count
                                                             ' 1
                        wrbyte  asmCount, asmRegAddr         ' 7  set count
                        'get the data we are transmitting
                        'initial dataformat: numUartDataL_nmUartDataH = 76543210_Sxxxxxxx (S=startbit)
                        mov     asmRegAddr, asmRegBase       ' 4
                        add     asmRegAddr, #def#nmUartDataH ' 4
                                                             ' 1
                        rdword  asmOutput, asmRegAddr        ' 7  get word[bank+nmUartDataH]
                        shr     asmOutput, #1                ' 4  bit to output now in bit6, stopbit moved into bit15
              if_z      mov     asmOutput, #0                ' 4  clear data if last bit to send (so word nmUartDataH becomes 0)
                                                             ' 1
                        wrword  asmOutput, asmRegAddr        ' 7  update word[bank+nmUartDataH]
                        'get invert state
                        add    asmRegAddr,#def#nmUartInvert - def#nmUartDataH ' 4
                        nop                                  ' 4
                                                             ' 1
                        rdbyte  asmInvert, asmRegAddr        ' 7  get invert state
                        and     asmInvert, #1 wz             ' 4
              if_z      xor     asmOutput, #$40              ' 4 if invert==1 invert outputbit
                        and     asmOutput, #$40 wz           ' 4  extract bit to output (bit6)
                        'output bit
              if_nz     or      vpOutLow, asmOutMask         ' 4  if bit==1 make output low
              if_z      or      vpOutHigh, asmOutMask        ' 4  if bit==0 make output high
vpUartTxAsm_ret         jmp     #isrAsm_retFromVP            ' 4  done


'variables to monitor 8.68 usec tick
time                    long    0
period                  long    694             '694 ticks at 80MHz equals 8.68 usec

'VP code global registers
asmVpType               long    0
asmTimerCountAddr       long    0               'address of 32bit free running timer
asmJumpAddr             long    0               'address to jump to
asmVpRambankArea        long    0               'start of vpRambank area used in this COG
asmBankOffset           long    0               'increments with 16
asmRegBase              long    0               'base for VP registers
asmRegAddr              long    0               'currently accessed VP register
adrInmask               long    0               'holds address of inmask
adrOutmask              long    0               'holds address of outmask
vpSlotCount             long    0

'in/out buffers
vpInput                 long    0               'sampled inputs
vpIn                    long    0               'bits set make corresponding pins inputs
vpOutLow                long    0               'bits set make corresponding pins low outputs
vpOutHigh               long    0               'bits set make corresponding pins high outputs

'vpTimer,vpAdc,vpDac,vpPwm,vpUartRx,vpUartTx temporary registers
asmAccu                 long    0
asmCount                long    0
asmValue                long    0
asmState                long    0
asmInMask               long    0
asmOutMask              long    0
asmInput                long    0
asmOutput               long    0
asmDivide               long    0
asmInvert               long    0
asmBaud                 long    0

vpCoreJumpTable         long    (@vpNoneAsm   - @entry)/4 '0
                        long    (@vpTimerAsm  - @entry)/4 '1
                        long    (@vpAdcAsm    - @entry)/4 '2
                        long    (@vpDacAsm    - @entry)/4 '3
                        long    (@vpPwmAsm    - @entry)/4 '4
                        long    (@vpUartRxAsm - @entry)/4 '5
                        long    (@vpUartTxAsm - @entry)/4 '6

                        fit     $1F0
  